package uploadsrcdir

import (
    "path/filepath"
    "fmt"
    "io/fs"
    "os"
    "runtime"
    "slices"
    "strings"
    "sync"
    "time"
    "gitea.armuli.eu/museum-digital/museum-digital-webdav-uploader/src/configloader"
)

var MetadataFileExts = []string{".xml", ".csv", ".json"}
var MediaFileExts = []string{".jpg", ".jpeg", ".png", ".gif", ".bmp", ".tif", ".tiff", ".pdf", ".mp3", ".mp4"}

// Lists all files in a folder that are of any of the expected file extensions.
func getFilesInDir(startPath string, fileExts []string) []os.FileInfo {

	allFiles := []os.FileInfo{}
	filepath.WalkDir(startPath, func(path string, entry fs.DirEntry, err error) error {

        if err != nil {
            return err
        }

        if slices.Contains(fileExts, filepath.Ext(entry.Name())) || slices.Contains(fileExts, strings.ToLower(filepath.Ext(entry.Name()))) {
            // add entry and it's size to dirMap
            file, err := entry.Info()
            if err != nil {
                fmt.Println("Error retrieving file info for:", err)
            } else {
                allFiles = append(allFiles, file)
            }
        }

        return nil

	})

    return allFiles

}

// Lists all files to upload in metadata directory
func ListUploadableMetadata(config configloader.MDWebDavUploaderConfig) []os.FileInfo {

    return getFilesInDir(config.UploadDir, MetadataFileExts);

}

// Lists all files to upload in metadata directory
func ListUploadableMedia(config configloader.MDWebDavUploaderConfig) []os.FileInfo {

    return getFilesInDir(config.UploadDir, MediaFileExts);

}

// Returns lists of uploadable folder contents grouped into a) metadata files and b) media files.
func GetUploadableFiles(config configloader.MDWebDavUploaderConfig) ([]string, []string) {

    metadataFiles := []string{}
    mediaFiles := []string{}

	filepath.WalkDir(config.UploadDir, func(path string, entry fs.DirEntry, err error) error {

        if err != nil {
            return err
        }

        if slices.Contains(MetadataFileExts, filepath.Ext(entry.Name())) || slices.Contains(MetadataFileExts, strings.ToLower(filepath.Ext(entry.Name()))) {
            metadataFiles = append(metadataFiles, path)
        }

        if slices.Contains(MediaFileExts, filepath.Ext(entry.Name())) || slices.Contains(MediaFileExts, strings.ToLower(filepath.Ext(entry.Name()))) {
            mediaFiles = append(mediaFiles, path)
        }

        return nil

	})

    return metadataFiles, mediaFiles

}

// Checks if a list of files have been modified in the last minute.
// If so, returns true.
func CheckAnyFileIsTooRecent(files []string) bool {

    now := time.Now()
    maxConcTasks := min(10, runtime.NumCPU())

    // Set a semaphore to restrict the number of concurrent upload tasks.
    semaphore := make(chan struct{}, maxConcTasks)
    wg := &sync.WaitGroup{}

    output := false

    for _, f := range(files) {

        semaphore <- struct{}{} // acquire
        wg.Add(1)

        go func() {

            defer wg.Done()
            file, fStatErr := os.Stat(f)
            if fStatErr != nil {
                panic("Failed to read file: " + f)
            }

            if diff := now.Sub(file.ModTime()); diff < 1 * time.Minute {
                print("File " + file.Name() + " has been changed within the last minute\n")
                output = true
            }

            <-semaphore // release

        }()

    }

    wg.Wait()

    return output

}
